//*****************************************************************************************************************************************
// TITLE: 		W2CALENDAR - A pop-up Modal calendar to select a date and/or enter a time. 
//
// Version	Date			Comments
// -------  ----------		--------------------------------------------------------------------------------------------------------------
// V1.0		23/10/2017		Initial release
// V1.1     27/10/2017      Conversion to Revealing Module design pattern and inclusion of configuration setup.
// V1.2     08/11/2017      Fixed bug with the today's date which was less than 2 characters. Adjusted highlight of selected date.
//
//*****************************************************************************************************************************************
// USE:
//
// 1. Update the default configuration settings
// 2. Add the following to your HTML page, in between the <head></head> tags, updating the pathname as appropriate:
//
// 		<link rel="stylesheet" type="text/css" href="css/w2calendar.css" />
// 		<script type="text/javascript" src="js/w2calendar.js"></script>
//
// 3. Add <input> fields to your form, as per your design.
// 4. Add the following at the end of the <body> section, but before </body> tag:
//
// 		<script type="text/javascript">
// 			window.onload = function () { 
//   			w2Calendar.init();
//	 			w2Calendar.insert('timeIB', { returnFormat: 'HH:MM' });
//   			w2Calendar.insert('dateIB', { returnFormat: 'DD-MM-YYYY' });
// 			}
// 		</script>
//
//*****************************************************************************************************************************************
// LICENSE: 
//
// w2Calendar (c) by Mark Naughton (blueshantung.uk)
//
// This is licensed under a Creative Commons Attribution-NoDerivatives 4.0 International License.
// You should have received a copy of the license along with this work. If not, see <http://creativecommons.org/licenses/by-nd/4.0/>.
//
//*****************************************************************************************************************************************

var w2Calendar = function() {
	'use strict';
	var monthsYear, fd, pMonth, pYear, showDate, showTime, formatDate, selectedDate, dayWeekNumbers,
	
//*****************************************************************************************************************************************
// Configuration variables - these are the defaults and can be updated here.....;
//
// These are the default configuration. They can be changed here, or via the insert() function call. 
//
// Value					Description
// ----------------------------------------------------------------------------------------------------------------------------------------
// Id						ID of the <div> for the Calendar.
// returnFormat				The format for the date/time. Values can be: DD/MM/YYYY, YYYY/MM/DD, DD-MM-YYYY or YYYY-MM-DD. For just the 
//                          time, you can use HH:MM or append to one of the Date formats. 
// useCurrentTime			(true/false) Specify if the Calendar will show the current date/time if the input box is blank.
// weekStarts				The starting day of the week, ie Monday - it must be the full word
// monthsYear				Names of each month :-)
// daysWeek					List of days in the week - Sunday must always be specified first as Javascript uses this. 
// defaultDayOrdinal		The ordinal for dates, ie "th" as in 13th.
// dayOrdinals 				A list of different ordinals for dates.
// disableWeekends			(true/false) Allow the user to select weekends as valid dates.
// disableWeekendsBgColor	Background color used when weekends are disabled.
// disableWeekendsTextColor	Text color when weekends are disabled.
// altTodayString			String for alternate text when user hovers over the Today icon.
// selectDateTitle			Title string for selecting a date.
// timeOnlyTitle			Title string for selecting a time.
// invalid24Error			Error message if user selects time > 24:00.
// timeRegionError			Error message if user selects a time outside of 00:00 to 23:59.
// loggingEnabled			(true/false) Writes logging messages to the console.
// mainBorderRadius			Border radius (in px) for the main <div> - I have found this needs to be greater than the contentBorderRadius.
// contentBorderRadius		Border radius (in px) for the content <div>.
// dropShadow				Add a drop shadow to the Calendar. Normal CSS applies (horizontal shadow, vertical shadow, blur, color of the shadow)
// headerBgColor			Background color of the Header
// headerTextColor			Text color of the Header
// errorBgColor				Background color when an error is shown
// errorTextColor			Text color of the error text
// monthNameBgColor			Background color of the Month name
// monthNameTextColor		Text color of the Month name
// timeBgColor				Background color of the Time input fields
// dayNameBgColor			Background color of the Day names
// dayNameTextColor			Text color of the Day names
// dateTextColor			Text color of the Dates
// dateBgColor				Background color of the Dates
// selectedBgColor			Background color when a Date is selected
// selectedTextColor		Text color when a Date is selected
// okButtonText				Text of the positive confirmation button
// cancelButtonText			Text of the negative confirmation button
// buttonBgColor			Background color of the buttons
// buttonTextColor			Text color of the buttons
// leftArrowImage			Path and filename of the image to select a previous month
// rightArrowImage			Path and filename of the image to select the next month
// timeImage				Path and filename of the image highlighting the Time input fields
// errorImage				Path and filename of the image indicating an error
// todayImage				Path and filename of the image used by the user to select the current date/time 
// 
//*****************************************************************************************************************************************
	configuration = { 
		Id:							'ModalCalendar',
		returnFormat:				'DD-MM-YYYY HH:MM',
		useCurrentTime:				true,
		weekStarts:					'Monday', 
		monthsYear:					[ "January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December" ],
		daysWeek:					[ "Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday" ],
		defaultDayOrdinal:			'th',
		dayOrdinals: 				[ [ "1", "st" ], [ "2", "nd" ], [ "3", "rd" ], [ "21", "st" ], [ "22", "nd" ], [ "23", "rd" ], [ "31", "st" ] ],
		disableWeekends:			false,
		disableWeekendsBgColor:		'#ccc',
		disableWeekendsTextColor:	'#999',
		altTodayString:				'Today',
		selectDateTitle:			'Select a date',
		timeOnlyTitle:				'Select a time',
		invalid24Error:				'Invalid time - only 24:00 is valid',
		timeRegionError:			'Time must be within 00:00 to 23:59',
		loggingEnabled:				true,
		mainBorderRadius:			'15px',
		contentBorderRadius:		'10px',
		dropShadow:					'10px 10px 5px #888',
		headerBgColor:				'#78909c',		
		headerTextColor:			'#fff',
		errorBgColor:				'#ffd42a',
		errorTextColor:				'#444',
		monthNameBgColor:			'#f0f0f0',
		monthNameTextColor:			'#444',
		timeBgColor:				'#4b636e',
		dayNameBgColor:				'#f0f0f0',
		dayNameTextColor:			'#555',
		dateTextColor:				'#aaa',
		dateBgColor:				'#f0f0f0',
		selectedBgColor:			'#90a4ae',
		selectedTextColor:			'#fff',
		okButtonText:				'Ok',
		cancelButtonText:			'Cancel',
		buttonBgColor:				'#f0f0f0',
		buttonTextColor:			'#006064',	
		leftArrowImage:				'images/ic_keyboard_arrow_left_black_24dp_1x.png',	
		rightArrowImage:			'images/ic_keyboard_arrow_right_black_24dp_1x.png',
		timeImage:					'images/ic_access_time_white_24dp_1x.png',
		errorImage:					'images/ic_error_outline_black_24dp_1x.png',
		todayImage:					'images/ic_today_black_24dp_1x.png'
	},

//*****************************************************************************************************************************************
// Routine:		init
// Exposure:	PUBLIC
//
// Function:	Initialise calendar by appending to the end of the HTML body
// Parameters:	none
//
//*****************************************************************************************************************************************
	init = function() {
		var newHTML = document.createElement ('div');
		newHTML.innerHTML = '<div id="'+configuration.Id+'" class="w2c-modal-calendar"></div>';
		document.body.appendChild(newHTML);
	},
	
//*****************************************************************************************************************************************
// Routine:		insert
// Exposure:	PUBLIC
//
// Function:	Add main routine as a onclick() mechanism to the named field, with specified config changes 
// Parameters:	dataID (ID of the input element to be assigned with the onclick())
//  			configChanges (List of config changes to be applied to this input element)
//
//*****************************************************************************************************************************************
	insert = function(dataID, configChanges) {
		var element = document.getElementById(dataID);
		element.addEventListener("click", function(e) {
			_w2Calendar(dataID,configChanges);
		}, false);
	},

//*****************************************************************************************************************************************
// Routine:		_w2Calendar
// Exposure:	PRIVATE
//
// Function:	Print calendar 
// Parameters:	showFirst (highlight the 1st day of the month)
//
//*****************************************************************************************************************************************
	_w2Calendar = function(inputID, configChanges) {

// Update the configuration from any specified on the function call line	
		_updateConfig(configChanges);
// Use the configuration to set the first day of the week		
		_setStartOfWeek();
	
		showDate = false;
		showTime = false;
		formatDate = configuration.returnFormat;

// Show and build the calendar	
		document.getElementById(configuration.Id).style.display = "block"
		var str = "";
		str += '<div class="w2c-modal-calendar-content">\n';
		str += '	<div id="fullDay" class="w2c-full-date"></div>\n';
		str += '	<div id="showTime"></div>\n';
		str += '	<div id="showDate"></div>\n';
		str += '	<div id="buttons" class="w2c-area-button"></div>\n';
		str += '</div>\n';
		document.getElementById(configuration.Id).innerHTML = str;
		
// Style the shadow and borders		
		document.getElementsByClassName("w2c-modal-calendar-content")[0].style.boxShadow = configuration.dropShadow;
		document.getElementsByClassName("w2c-modal-calendar-content")[0].style.borderRadius = configuration.mainBorderRadius;
		document.getElementsByClassName("w2c-full-date")[0].style.borderTopLeftRadius = configuration.contentBorderRadius;
		document.getElementsByClassName("w2c-full-date")[0].style.borderTopRightRadius = configuration.contentBorderRadius;
		document.getElementsByClassName("w2c-area-button")[0].style.borderBottomLeftRadius = configuration.contentBorderRadius;
		document.getElementsByClassName("w2c-area-button")[0].style.borderBottomRightRadius = configuration.contentBorderRadius;

// Find what formats the user wants	
		if(formatDate.indexOf("DD/MM/YYYY") > -1) { showDate = true; }
		if(formatDate.indexOf("DD-MM-YYYY") > -1) { showDate = true; }
		if(formatDate.indexOf("YYYY-MM-DD") > -1) { showDate = true; }
		if(formatDate.indexOf("YYYY/MM/DD") > -1) { showDate = true; }
		if(formatDate.indexOf("HH:MM") > -1) { showTime = true; }

//Show the time input 
		if(showTime) {	
			_showTime(showDate);
		}
// Show the Month input
		if(showDate) {	
			_showDate();
			var dt_date = new Date();
			pMonth = dt_date.getMonth();
			pYear = dt_date.getFullYear();
			_calendarDraw(false);
		}
// Show the buttons
		_showButtons();
		
//Change the Calendar header if the user just wants time
		if(showTime && !showDate) {
			document.getElementById('fullDay').innerHTML = configuration.timeOnlyTitle;
			document.getElementById('fullDay').style.backgroundColor = configuration.headerBgColor;		
			document.getElementById('fullDay').style.color = configuration.headerTextColor;
		} else {
			document.getElementById('fullDay').style.backgroundColor = configuration.headerBgColor;		
			document.getElementById('fullDay').style.color = configuration.headerTextColor;
		}
// Select today's date/time if the user has supplied a blank input, otherwise use that to show	
		if(document.getElementById(inputID).value != "") {
			_setDateTime(document.getElementById(inputID).value);
		} else {
			if(configuration.useCurrentTime) {
				_todayDate();
			} else {
				if(showTime && !showDate) {
					document.getElementById('fullDay').innerHTML = configuration.timeOnlyTitle;	
				} else {
					document.getElementById('fullDay').innerHTML = configuration.selectDateTitle;
				}
				if(showTime) {
					document.getElementById('hours').value = "";
					document.getElementById('mins').value = "";
				}
			}
		}
// Set up onclick for the Ok button - if the user sets a valid date/time, copy the new date/time into the Input box and close the Calendar	
		document.getElementById("ok").onclick = function(event) {
			var rs = _validateData();
			if(rs != "ERROR") {
				document.getElementById(inputID).value = rs;
				document.getElementById(configuration.Id).style.display = "none";
			}
		}
// Close the Calendar when the user clicks Cancel
		document.getElementById("cancel").onclick = function(event) {
			document.getElementById(configuration.Id).style.display = "none";
		}
	},	
//*****************************************************************************************************************************************
// Routine:		_validateData
// Exposure:	PRIVATE
//
// Function:	Depending on the required format, the data is either returned or validated first. If the time values are not correct,
//              then an error is displayed in the Calendar header.
// Parameters:	none
//
//*****************************************************************************************************************************************
	_validateData = function() {
		var returnedValue = formatDate;
		
		if(showTime) {
// get the format of the time and the time values from the form
			var hpos = formatDate.indexOf("HH:MM");
			var hh = _leadingZero(document.getElementById('hours').value);
			var mm = _leadingZero(document.getElementById('mins').value);
// Validate the hours and minutes - report an error and change the form to show 
			var str = "";
			if(parseInt(hh) == 24 && parseInt(mm) != 0) {
				document.getElementById('fullDay').innerHTML = configuration.invalid24Error;
				document.getElementById('fullDay').className = "w2c-show-errors";
				document.getElementById('fullDay').style.color = configuration.errorTextColor;
				document.getElementById('fullDay').style.backgroundColor = configuration.errorBgColor;
				document.getElementById('fullDay').style.backgroundImage = "url("+configuration.errorImage+")";
				return "ERROR"; 
			}
			if(((parseInt(hh) < 0 || parseInt(hh) > 23) || (parseInt(mm) < 0 || parseInt(mm) > 59)) && (parseInt(hh) != 24 && parseInt(mm) != 0)) {	
				document.getElementById('fullDay').innerHTML = configuration.timeRegionError; 
				document.getElementById('fullDay').style.color = configuration.errorTextColor;
				document.getElementById('fullDay').style.backgroundColor = configuration.errorBgColor;
				document.getElementById('fullDay').style.backgroundImage = "url("+configuration.errorImage+")";
				document.getElementById('fullDay').className = "w2c-show-errors";
				return "ERROR"; 
			}
// Else return the new time entered
			returnedValue = returnedValue.replace("HH:MM",hh+":"+mm);
		}
		if(showDate) {
// Get the month, day and year and the format and return
			var mth = _leadingZero(selectedDate[1]);
			var day = _leadingZero(selectedDate[2]);
			var year = selectedDate[0];
			if(formatDate.indexOf("DD-MM-YYYY") > -1) { returnedValue = returnedValue.replace("DD-MM-YYYY",day+"-"+mth+"-"+year); }
			if(formatDate.indexOf("DD/MM/YYYY") > -1) { returnedValue = returnedValue.replace("DD/MM/YYYY",day+"/"+mth+"/"+year); }
			if(formatDate.indexOf("YYYY-MM-DD") > -1) { returnedValue = returnedValue.replace("YYYY-MM-DD",year+"-"+mth+"-"+day); }
			if(formatDate.indexOf("YYYY/MM/DD") > -1) { returnedValue = returnedValue.replace("YYYY/MM/DD",year+"/"+mth+"/"+day); }
		}
		return returnedValue;
	},

//*****************************************************************************************************************************************
// Routine:		_selectDate
// Exposure:	PUBLIC
//
// Function:	Clear the dates of any selection, then select the new date and highlight
// Parameters:	element (date to highlight)
//
//*****************************************************************************************************************************************
	_selectDate = function(element) {
// Get the ID of the date selected
		var dateID = element.id;
// Clear all existing dates of highlights
		var c = document.getElementsByClassName("w2c-day");
		for(var i=0; i < c.length; i++) {
			if(c[i].className == "w2c-day") {
// Get the ID and the weekday number (Sunday=0..)				
				var dID = c[i].id;
				var components = dID.split('-');
				var componentDay = parseInt(components[3]);
// If the weekends are disabled, then use the configuration colors for the disabledWeekend, and clear the borderRadius in case the date falls at the weekend				
				if(configuration.disableWeekends && (componentDay == 0 || componentDay == 6)) {
					c[i].style.color = configuration.disableWeekendsTextColor;
					c[i].style.backgroundColor = configuration.disableWeekendsBgColor;
					c[i].style.borderRadius = "0px";					
				} else {
// Style the day with standard colors					
					c[i].style.color = configuration.dateTextColor;
					c[i].style.backgroundColor = configuration.dateBgColor;
				}
			}
		}
// If the selected date is not highlighted, then highlight it and then update the header with the English date name
		if(document.getElementById(dateID).style.backgroundColor  = configuration.dateBgColor) {
			document.getElementById(dateID).style.color = configuration.selectedTextColor;;
			document.getElementById(dateID).style.backgroundColor  = configuration.selectedBgColor; 
			document.getElementById(dateID).style.borderRadius  = "50%"; // "10px"; 
			_drawFullDate(dateID);
		}
	},

//*****************************************************************************************************************************************
// Routine:		_drawFullDate
// Exposure:	PRIVATE
//
// Function:	Use the selected date and display the full day, date, month and year in the header 
// Parameters:	newID (ID of the selected date)
//
//*****************************************************************************************************************************************
	_drawFullDate = function(newID) {
// Split the ID of the selected date	
		selectedDate = newID.split('-');
// Get the month and day	
		var newMonth = selectedDate[1] - 1;
		var newDay = selectedDate[2];
		if(selectedDate[2].charAt(0) == '0') { newDay = selectedDate[2].charAt(1); }
// Search for the correct suffix to add	
		var found = false;
		for(var j=0; j < configuration.dayOrdinals.length; j++) {
			if(configuration.dayOrdinals[j][0] == newDay) { newDay = newDay+"<small>"+configuration.dayOrdinals[j][1]+"</small>"; found = true; break; }
		}
// Use the default if not found in the stem 
		if(!found) { newDay = newDay+"<small>"+configuration.defaultDayOrdinal+"</small>"; }
// Build the date and update the header
		var dateString = configuration.daysWeek[selectedDate[3]]+" "+newDay+" "+configuration.monthsYear[newMonth]+" "+selectedDate[0];
		document.getElementById('fullDay').className = "w2c-full-date";
		document.getElementById('fullDay').innerHTML = dateString;
		document.getElementById('fullDay').style.backgroundColor = configuration.headerBgColor;		
		document.getElementById('fullDay').style.color = configuration.headerTextColor;
		document.getElementById('fullDay').style.backgroundImage = "none";
	},

//*****************************************************************************************************************************************
// Routine:		_todayDate
// Exposure:	PUBLIC
//
// Function:	Reset calendar to show today's date, highlight the date and set the time
// Parameters:	none
//
//*****************************************************************************************************************************************
	_todayDate = function() {
// Get today's date and update the calendar as we might be displaying another
		var td = new Date();
		if(td.getMonth() != pMonth) {
			pMonth = td.getMonth();
			pYear = td.getFullYear();
			_calendarDraw();
		}
// If the user wants the date, build the ID and then highlight it
		if(showDate) {
			var todayMonth = td.getMonth() + 1;
			var todayID = td.getFullYear()+"-"+_leadingZero(todayMonth)+"-"+_leadingZero(td.getDate())+"-"+td.getDay();
			_outputLog(todayID);
			_selectDate(document.getElementById(todayID));
		}
// If the user wants the time, insert into the Time input fields
		if(showTime) {
			document.getElementById('hours').value = _leadingZero(td.getHours());
			document.getElementById('mins').value = _leadingZero(td.getMinutes());
		}
	},

//*****************************************************************************************************************************************
// Routine:		_setDateTime
// Exposure:	PRIVATE
//
// Function:	Set the Calendar to show the selected date and time 
// Parameters:	value (date and/or time from the input box)
//
//*****************************************************************************************************************************************
	_setDateTime = function(value) {
		var actualDate = value.toString();
		var sDay, sMonth, sYear;
		
// If the user wants the date, get the date from the value, parse, draw the calendar and set the ID and then highlight it
		if(showDate) {
			sDay = actualDate.substr(0,2);
			if(formatDate.indexOf("DD-MM-YYYY") > -1 || formatDate.indexOf("DD/MM/YYYY") > -1) {
				sMonth = parseInt(actualDate.substr(3,2)) - 1;
				sYear = parseInt(actualDate.substr(6,4));
			}
			if(formatDate.indexOf("YYYY-MM-DD") > -1 || formatDate.indexOf("YYYY/MM/DD") > -1) {
				sMonth = parseInt(actualDate.substr(5,2)) - 1;
				sYear = parseInt(actualDate.substr(0,4));
			}
			pMonth = sMonth;
			pYear = sYear;
			_calendarDraw(false);
			var td = new Date(pYear,pMonth,sDay);
			var todayMonth = _leadingZero(pMonth + 1);
			var todayID = pYear+"-"+todayMonth+"-"+sDay+"-"+td.getDay();
			_selectDate(document.getElementById(todayID));
		}
// If the user wants the time, get the time from the value and insert into the Time input fields	
		if(showTime) { 
			var i = formatDate.indexOf("HH:MM");
			document.getElementById('hours').value = _leadingZero(actualDate.substr(i,2));
			document.getElementById('mins').value = _leadingZero(actualDate.substr(i+3,2));
		}
	},

//*****************************************************************************************************************************************
// Routine:		_leadingZero
// Exposure:	PRIVATE
//
// Function:	Check the length and add leading zero 
// Parameters:	value (value to check)
//
//*****************************************************************************************************************************************
	_leadingZero = function(value) {
		var returned = value.toString();
// Check length of the value - if one, it needs a leading zero added and then return	
		if(returned.length == 1) { returned = "0" + returned; }
		return returned;
	},

//*****************************************************************************************************************************************
// Routine:		_calendarDraw
// Exposure:	PRIVATE
//
// Function:	Print calendar 
// Parameters:	showFirst (highlight the 1st day of the month)
//
//*****************************************************************************************************************************************
	_calendarDraw = function(showFirst) {
		var show_date = true, wDay, str = '';

// If the user does not want the Date, return (just in case we get here by mistake)
		if(!showDate) { return; }
		
// Get the selected Date
		fd = new Date(pYear,pMonth);
// Validate the month and year to be displayed	
		var monId = parseInt(fd.getMonth());	// Month returned as 0 to 11
		document.getElementById("monthName").innerHTML = configuration.monthsYear[monId]+" "+fd.getFullYear();
		monId++;
		var w2cYear = pYear;
		if(monId > 12) { monId = 1; w2cYear++; }
		var dateID = w2cYear+"-"+_leadingZero(monId);		// Used as main part in the ID tag
		var firstDay = fd.getDay();							// Get Weekday of the first of the month

// Show the Day headers
		_calendarDays();
// Loop six rows, with 7 days and then draw the Days - adding Onclick() for selection. Also add blank days.
		for (var n_rows=0; n_rows < 6; n_rows++) {
			str += '<div style="width:100%; text-align: center;">\n';
			for (var dW=0; dW < dayWeekNumbers.length; dW++) {	
				wDay = dayWeekNumbers[dW];
				if(show_date) {
					if(fd.getDay() == wDay) { 
						var newID = dateID+'-'+_leadingZero(fd.getDate())+'-'+wDay;
						str += '<div ';
// Add the onclick() to the date if its a weekday and weekend (but not if user wants weekends disabled)						
						var makeSelect = true;
						if(configuration.disableWeekends && (fd.getDay() == 0 || fd.getDay() == 6)) { makeSelect = false; }
						if(makeSelect) {
							str += 'onclick="w2Calendar.selectDate(this);"';
						}
						str += ' id="'+newID+'" class="w2c-day">'+fd.getDate(); 
						if(_isLastDay(fd)) { show_date = false; }		// if we are the last day, set flag and continue
						fd.setDate(fd.getDate()+1);						// Step forward to next day
					} else { 
						str += '<div class="w2c-blank-day">&nbsp;'; 
					}
				} else {
					str += '<div class="w2c-blank-day">&nbsp;'; 
				}
				str += '</div>';
			}
			str += '</div>\n';
			if(show_date == false) { break; }		// If we are the last day, then break out of the loop
		}
		document.getElementById("days").innerHTML = str;		// Add to the HTML
// Color the days with the text and backgroundColor		
		var thisButtons = document.getElementsByClassName("w2c-day");
		for (var i = 0; i < thisButtons.length; i++) {
			if(!thisButtons[i].getAttribute("style") || thisButtons[i].getAttribute("style") == null || thisButtons[i].getAttribute("style") == "") {
				thisButtons[i].style.backgroundColor = configuration.dateBgColor;
				thisButtons[i].style.color = configuration.dateTextColor;
			}
		}
// If user has moved to a new month, we highlight the first day of that month	
		if(showFirst) {
			_selectDate(document.getElementById(dateID+"-01-"+firstDay))
		}
	},

//*****************************************************************************************************************************************
// Routine:		_showDate
// Exposure:	PRIVATE
//
// Function:	Displays the date and month selectors 
// Parameters:	none
//
//*****************************************************************************************************************************************
	_showDate = function() {
		var str = "";
		str += '<!-- Month Selector -->\n';
		str += '	<div class="w2c-area-month">\n';
		str += '		<img onclick="javascript:w2Calendar.calendarPreMonth();" src="'+configuration.leftArrowImage+'" border=0 class="w2c-nav-arrow">\n';
		str += '		<div id="monthName" class="w2c-area-month-name"></div>\n';
		str += '		<img onclick="javascript:w2Calendar.calendarNxtMonth();" src="'+configuration.rightArrowImage+'" border=0 class="w2c-nav-arrow">\n';
		str += '	</div>\n';
		str += '<!-- Day Selector -->\n';
		str += '	<div class="w2c-area-day-name">\n';
		str += '		<div id="dayHead" class="w2c-area-day-names"></div>\n';
		str += '	</div>\n';
		str += '	<div id="days" name="days" class="w2c-area-day"></div>\n';
		document.getElementById('showDate').innerHTML = str;
// Change the background color for the Month Name - only one so we don't need to loop		
		document.getElementsByClassName('w2c-area-month')[0].style.backgroundColor = configuration.monthNameBgColor;
		document.getElementById('monthName').style.color = configuration.monthNameTextColor;
	},

//*****************************************************************************************************************************************
// Routine:		_showTime
// Exposure:	PRIVATE
//
// Function:	Displays the input boxes for the Time 
// Parameters:	padTime (if the Calendar displays just the time, add a padding style)
//
//*****************************************************************************************************************************************
	_showTime = function(padTime) {
		var padBtm;
		if(!padTime) { padBtm = 'style="padding-bottom: 10px;"'; }
		var str = "";
		str += '	<div class="w2c-area-time" '+padBtm+'>\n';
		str += '		<img src="'+configuration.timeImage+'" border=0 class="w2c-time-icon">\n';
		str += '		<input type="number" id="hours" required class="w2c-time-range" min="0" max="24"> : <input type="number" id="mins" required class="w2c-time-range" min="0" max="59">\n';
		str += '	</div>\n';
		document.getElementById('showTime').innerHTML = str;
// Change the background color for the Time - only one so we don't need to loop		
		document.getElementsByClassName('w2c-area-time')[0].style.backgroundColor = configuration.timeBgColor;
	},

//*****************************************************************************************************************************************
// Routine:		_showButtons
// Exposure:	PRIVATE
//
// Function:	Displays the buttons on the Calendar
// Parameters:	none
//
//*****************************************************************************************************************************************
	_showButtons = function() {
		var str = "";
		str += '	<img onclick="javascript:w2Calendar.todayDate();" src="'+configuration.todayImage+'" title="'+configuration.altTodayString+'" border=0 class="w2c-today-icon">\n';
		str += '	<button id="ok" type="button" class="w2c-button">'+configuration.okButtonText+'</button> <button id="cancel" type="button" class="w2c-button">'+configuration.cancelButtonText+'</button>&nbsp;&nbsp;\n';
		document.getElementById('buttons').innerHTML = str;
		
		var thisButtons = document.getElementsByClassName("w2c-button");
		for (var i = 0; i < thisButtons.length; i++) {
			thisButtons[i].style.backgroundColor = configuration.buttonBgColor;
			thisButtons[i].style.color = configuration.buttonTextColor;
		}
	},

//*****************************************************************************************************************************************
// Routine:		_calendarDays
// Exposure:	PRIVATE
//
// Function:	Display the names of each day in the header
// Parameters:	none
//
//*****************************************************************************************************************************************
	_calendarDays = function() {
		var str = "", dW;
		for(var i=0; i < dayWeekNumbers.length; i++) {
			dW = dayWeekNumbers[i];
			str += '<div class="w2c-day-name">'+configuration.daysWeek[dW].substring(0,2)+'</div>\n';
		}
		document.getElementById('dayHead').innerHTML = str;
		var thisButtons = document.getElementsByClassName("w2c-day-name");
		for (var i = 0; i < thisButtons.length; i++) {
			thisButtons[i].style.backgroundColor = configuration.dayNameBgColor;
			thisButtons[i].style.color = configuration.dayNameTextColor;
		}		
	},

//*****************************************************************************************************************************************
// Routine:		_setStartOfWeek
// Exposure:	PRIVATE
//
// Function:	Using the 'weekStarts' definition, this routine parses the list of day names, and builds the correct order of days using 
//              their Javascript number - Sunday is always 0, regardless of Region.
// Parameters:	none
//
//*****************************************************************************************************************************************
	_setStartOfWeek = function() {
		var jsDayNumbers = [ 0,1,2,3,4,5,6 ];
		var found = false, dW = -1;
		
		dayWeekNumbers = new Array();
		for(var i=0; i < jsDayNumbers.length; i++) {
			if(configuration.weekStarts == configuration.daysWeek[i]) {
				found = true;
			}
			if(found) {
				dayWeekNumbers[++dW] = jsDayNumbers[i];
				jsDayNumbers[i] = -1;
			}
		}
		for(var i=0; i < jsDayNumbers.length; i++) {
			if(jsDayNumbers[i] > -1) {
				dW++;
				dayWeekNumbers[dW] = jsDayNumbers[i];			
			}
		}	
	},
	
	
//*****************************************************************************************************************************************
// Routine:		calendarPreMonth
// Exposure:	PRIVATE
//
// Function:	Decrement the month, adjust the year and draw the Calendar
// Parameters:	none
//
//*****************************************************************************************************************************************
	calendarPreMonth = function() {	
		pMonth--;
		if(pMonth < 0) { pMonth = 11; }
		if(pMonth == 11) { pYear--; }
		if(pYear < 1900) { pYear += 1900; }
		_calendarDraw(true);
	},

//*****************************************************************************************************************************************
// Routine:		_calendarNxtMonth
// Exposure:	PRIVATE
//
// Function:	Increment the month, adjust the year and draw the Calendar
// Parameters:	none
//
//*****************************************************************************************************************************************
	calendarNxtMonth = function() {		
		pMonth++;
		if(pMonth > 11) { pMonth = 0; pYear++; }	
		if(pYear < 1900) { pYear += 1900; }
		_calendarDraw(true);
	},

//*****************************************************************************************************************************************
// Routine:		_isLastDay
// Exposure:	PRIVATE
//
// Function:	Detects if the date is the last day of the month 
// Parameters:	dt (Date value to be checked)
//
//*****************************************************************************************************************************************
	_isLastDay = function(dt) {
		return new Date(dt.getTime() + 86400000).getDate() === 1;
	},

//*****************************************************************************************************************************************
// Routine:		_outputLog
// Exposure:	PRIVATE
//
// Function:	Writes logging information and displayed in a div 
// Parameters:	sData (message to be written)
//
//*****************************************************************************************************************************************
	_outputLog = function(sData) {
		if(configuration.loggingEnabled) {
			console.log(sData);
		}
	},
	
//*****************************************************************************************************************************************
// Routine:		_updateConfig
// Exposure:	PRIVATE
//
// Function:	Updates the configuration with new values supplied by user on insert() call 
// Parameters:	configChanges (changes to configuration)
//
//*****************************************************************************************************************************************
	_updateConfig = function(configChanges) {
		for (var key in configChanges) {
			if (configuration.hasOwnProperty(key)) {
				configuration[key] = configChanges[key];
			}
		}
		for (var key in configuration) {
			if (configuration.hasOwnProperty(key)) {
				_outputLog(key + " -> " + configuration[key]);
			}
		}
	};
	
//*****************************************************************************************************************************************
// Routine:		RETURN
// Exposure:	PRIVATE/PUBLIC
//
// Function:	Not actually a routine, but reveals the public pointers to private functions
// Parameters:	none
//
//*****************************************************************************************************************************************
	return {
		init:init,
		insert:insert,
		todayDate:_todayDate,
		selectDate:_selectDate,
		calendarPreMonth:calendarPreMonth,
		calendarNxtMonth:calendarNxtMonth
	};
	
//*****************************************************************************************************************************************
// End of W2CALENDAR Function & Routines
//*****************************************************************************************************************************************
}();
